import { DeepClient, SerialOperation } from '@deep-foundation/deeplinks/imports/client';
import { PushNotification } from './push-notification';
/**
  * Gets serial operations to insert {@link Package.PushNotification}
  *
  * @example
  * #### Insert {@link Package.PushNotification}
 ```ts
 const {serialOperations, linkIds} = await getPushNotificationInsertSerialOperations({
   deep
 });
 await deep.serial({
   operations: serialOperations
 })
 ```
   * #### Insert {@link Package.PushNotification} with reserved link id
  ```ts
  const reservedLinkIds = await deep.reserve(2);
  const pushNotificationLinkId = reservedLinkIds.pop();
  const containLinkId = reservedLinkIds.pop();
  
  const {serialOperations, linkIds} = await getPushNotificationInsertSerialOperations({
    deep,
    pushNotification: {
      title,
      body,
    },
    reservedLinkIds: {
      pushNotificationLinkId,
      containLinkId,
    }
  });
  await deep.serial({
    operations: serialOperations
  })
  ```
  */
export declare function getPushNotificationInsertSerialOperations(param: GetPushNotificationInsertSerialOperationsParam): Promise<GetPushNotificationInsertSerialOperationsResult>;
export interface GetPushNotificationInsertSerialOperationsParam {
    /**
     * Reserved link ids that will be used in the serial operations
     */
    reservedLinkIds?: {
        /**
         * Reserved link id for the pushNotification
         */
        pushNotificationLinkId?: number;
        /**
         * Reserved link id for the contain
         */
        containLinkId?: number;
    };
    /**
     * Link ids of types that will be used in the serial operations
     */
    typeLinkIds?: {
        /**
         * Link id of the contain type
         */
        containTypeLinkId?: number;
        /**
         * Link id of the pushNotification type
         */
        pushNotificationTypeLinkId?: number;
    };
    /**
     * Deep Client
     */
    deep: DeepClient;
    /**
     * PushNotification
     */
    pushNotification: PushNotification;
    /**
     * Link id of the container
     *
     * @remarks
     * If it is null, contain link will not be created
     * @defaultValue {@link GetPushNotificationInsertSerialOperationsParam.deep.linkId} if not provided or undefined
     */
    containerLinkId?: number | undefined | null;
    /**
     * Value of the contain link
     *
     * @remarks
     * If {@link GetPushNotificationInsertSerialOperationsParam.containerLinkId} is null, this will be ignored
     */
    containValue?: string | undefined;
}
export interface GetPushNotificationInsertSerialOperationsResult {
    serialOperations: Array<SerialOperation>;
    linkIds: Required<Exclude<GetPushNotificationInsertSerialOperationsParam['reservedLinkIds'], undefined>>;
}
