import { DeepClient, SerialOperation } from '@deep-foundation/deeplinks/imports/client';
/**
  * Gets serial operations to insert {@link Package.ServiceAccount}
  *
  * @example
  * #### Insert {@link Package.ServiceAccount}
 ```ts
 const {serialOperations, linkIds} = await getServiceAccountInsertSerialOperations({
   deep
 });
 await deep.serial({
   operations: serialOperations
 })
 ```
   * #### Insert {@link Package.ServiceAccount} with reserved link id
  ```ts
  const reservedLinkIds = await deep.reserve(2);
  const serviceAccountLinkId = reservedLinkIds.pop();
  const containLinkId = reservedLinkIds.pop();
  
  const {serialOperations, linkIds} = await getServiceAccountInsertSerialOperations({
    deep,
    serviceAccount: {
      title,
      body,
    },
    reservedLinkIds: {
      serviceAccountLinkId,
      containLinkId,
    }
  });
  await deep.serial({
    operations: serialOperations
  })
  ```
  */
export declare function getServiceAccountInsertSerialOperations(param: GetServiceAccountInsertSerialOperationsParam): Promise<GetServiceAccountInsertSerialOperationsResult>;
export interface GetServiceAccountInsertSerialOperationsParam {
    /**
     * Reserved link ids that will be used in the serial operations
     */
    reservedLinkIds?: {
        /**
         * Reserved link id for the serviceAccount
         */
        serviceAccountLinkId?: number;
        /**
         * Reserved link id for the contain
         */
        containForServiceAccountLinkId?: number;
        /**
         * Reserved link id for the usesServiceAccount
         */
        usesServiceAccountLinkId?: number;
        /**
         * Reserved link id for the contain for usesServiceAccount
         */
        containForUsesServiceAccountLinkId?: number;
    };
    /**
     * Link ids of types that will be used in the serial operations
     */
    typeLinkIds?: {
        /**
         * Link id of the contain type
         */
        containTypeLinkId?: number;
        /**
         * Link id of the serviceAccount type
         */
        serviceAccountTypeLinkId?: number;
        /**
         * Link id of the usesServiceAccount type
         */
        usesServiceAccountTypeLinkId?: number;
    };
    /**
     * Deep Client
     */
    deep: DeepClient;
    /**
     * Service Account
     */
    serviceAccount: string;
    /**
     * Link id of the container
     *
     * @remarks
     * If it is null, contain link will not be created
     * @defaultValue {@link GetServiceAccountInsertSerialOperationsParam.deep.linkId} if not provided or undefined
     */
    containerLinkId?: number | undefined | null;
    /**
     * Value for the contain link for {@link Package.ServiceAccount}
     *
     * @remarks
     * If {@link GetServiceAccountInsertSerialOperationsParam.containerLinkId} is null, this will be ignored
     */
    valueForContainForServiceAccount?: string | undefined;
    /**
     * Value for the contain link for {@link Package.UsesServiceAccount}
     *
     * @remarks
     * If {@link GetServiceAccountInsertSerialOperationsParam.containerLinkId} is null, this will be ignored
     */
    valueForContainForUsesServiceAccount?: string | undefined;
    /**
     * If true, the link will be made active by creating a {@link Package.UsesServiceAccount} link pointing to it
     *
     * @defaultValue false
     */
    shouldMakeActive?: boolean;
}
export interface GetServiceAccountInsertSerialOperationsResult {
    serialOperations: Array<SerialOperation>;
    linkIds: Required<Exclude<GetServiceAccountInsertSerialOperationsParam['reservedLinkIds'], undefined>>;
}
