import { createSerialOperation } from '@deep-foundation/deeplinks/imports/gql';
import { Package } from './package';
/**
  * Gets serial operations to insert {@link Package.ServiceAccount}
  *
  * @example
  * #### Insert {@link Package.ServiceAccount}
 ```ts
 const {serialOperations, linkIds} = await getServiceAccountInsertSerialOperations({
   deep
 });
 await deep.serial({
   operations: serialOperations
 })
 ```
   * #### Insert {@link Package.ServiceAccount} with reserved link id
  ```ts
  const reservedLinkIds = await deep.reserve(2);
  const serviceAccountLinkId = reservedLinkIds.pop();
  const containLinkId = reservedLinkIds.pop();
  
  const {serialOperations, linkIds} = await getServiceAccountInsertSerialOperations({
    deep,
    serviceAccount: {
      title,
      body,
    },
    reservedLinkIds: {
      serviceAccountLinkId,
      containLinkId,
    }
  });
  await deep.serial({
    operations: serialOperations
  })
  ```
  */
export async function getServiceAccountInsertSerialOperations(param) {
    const { deep, serviceAccount, valueForContainForServiceAccount, valueForContainForUsesServiceAccount, shouldMakeActive = false } = param;
    const $package = new Package({ deep });
    const containerLinkId = param.containerLinkId !== null ? param.containerLinkId ?? deep.linkId : null;
    const reservedLinkIds = await getReservedLinkIds();
    const { containForServiceAccountLinkId, serviceAccountLinkId, usesServiceAccountLinkId, containForUsesServiceAccountLinkId } = reservedLinkIds;
    const typeLinkIds = await getTypeLinkIds();
    const { containTypeLinkId, serviceAccountTypeLinkId, usesServiceAccountTypeLinkId } = typeLinkIds;
    const serialOperations = [];
    const serviceAccountInsertSerialOperation = createSerialOperation({
        type: 'insert',
        table: 'links',
        objects: {
            id: serviceAccountLinkId,
            type_id: serviceAccountTypeLinkId,
        },
    });
    serialOperations.push(serviceAccountInsertSerialOperation);
    const valueOfServiceAccountInsertSerialOperation = createSerialOperation({
        type: 'insert',
        table: 'objects',
        objects: {
            link_id: serviceAccountLinkId,
            value: serviceAccount,
        },
    });
    serialOperations.push(valueOfServiceAccountInsertSerialOperation);
    if (containerLinkId !== null) {
        const containInsertSerialOperation = createSerialOperation({
            type: 'insert',
            table: 'links',
            objects: {
                id: containForServiceAccountLinkId,
                type_id: containTypeLinkId,
                from_id: containerLinkId,
                to_id: serviceAccountLinkId,
            },
        });
        serialOperations.push(containInsertSerialOperation);
        if (valueForContainForServiceAccount) {
            const valueForContainInsertSerialOperation = createSerialOperation({
                type: 'insert',
                table: 'objects',
                objects: {
                    link_id: containForServiceAccountLinkId,
                    value: valueForContainForServiceAccount,
                },
            });
            serialOperations.push(valueForContainInsertSerialOperation);
        }
    }
    if (shouldMakeActive) {
        const usesServiceAccountInsertSerialOperation = createSerialOperation({
            type: 'insert',
            table: 'links',
            objects: {
                id: usesServiceAccountLinkId,
                type_id: usesServiceAccountTypeLinkId,
                from_id: deep.linkId,
                to_id: serviceAccountLinkId,
            },
        });
        serialOperations.push(usesServiceAccountInsertSerialOperation);
        if (containerLinkId !== null) {
            const containForUsesServiceAccountInsertSerialOperation = createSerialOperation({
                type: 'insert',
                table: 'links',
                objects: {
                    id: containForUsesServiceAccountLinkId,
                    type_id: containTypeLinkId,
                    from_id: containerLinkId,
                    to_id: usesServiceAccountLinkId,
                }
            });
            serialOperations.push(containForUsesServiceAccountInsertSerialOperation);
            if (valueForContainForUsesServiceAccount) {
                const valueForContainInsertSerialOperation = createSerialOperation({
                    type: 'insert',
                    table: 'objects',
                    objects: {
                        link_id: containForUsesServiceAccountLinkId,
                        value: valueForContainForUsesServiceAccount,
                    },
                });
                serialOperations.push(valueForContainInsertSerialOperation);
            }
        }
    }
    return {
        serialOperations,
        linkIds: reservedLinkIds
    };
    async function getReservedLinkIds() {
        let result = {
            containForServiceAccountLinkId: 0,
            serviceAccountLinkId: 0,
            usesServiceAccountLinkId: 0,
            containForUsesServiceAccountLinkId: 0
        };
        const linksToReserveCount = Object.keys(result).length -
            Object.keys(param.reservedLinkIds || {}).length;
        const reservedLinkIds = linksToReserveCount > 0 ? await deep.reserve(linksToReserveCount) : [];
        result = {
            containForServiceAccountLinkId: param.reservedLinkIds?.containForServiceAccountLinkId ?? reservedLinkIds.pop(),
            serviceAccountLinkId: param.reservedLinkIds?.serviceAccountLinkId ?? reservedLinkIds.pop(),
            usesServiceAccountLinkId: param.reservedLinkIds?.usesServiceAccountLinkId ?? reservedLinkIds.pop(),
            containForUsesServiceAccountLinkId: param.reservedLinkIds?.containForUsesServiceAccountLinkId ?? reservedLinkIds.pop(),
        };
        return result;
    }
    async function getTypeLinkIds() {
        const result = {
            containTypeLinkId: param.typeLinkIds?.containTypeLinkId ||
                (await deep.id('@deep-foundation/core', 'Contain')),
            serviceAccountTypeLinkId: param.typeLinkIds?.serviceAccountTypeLinkId ||
                await $package.ServiceAccount.id(),
            usesServiceAccountTypeLinkId: param.typeLinkIds?.usesServiceAccountTypeLinkId ||
                await $package.UsesServiceAccount.id(),
        };
        return result;
    }
}
